.login {
    width: 500px;
    padding: 8% 0 0;
    margin: auto;
    align-content: center;
}

.card {
    display: flex;
    justify-content: flex-end;
    border: none;
    transition: transform 0.3s ease, box-shadow 0.3s ease;
}

.draggable-method-card {
    display: flex;
    justify-content: flex-end;
    cursor: move;
    border: none;
    transition: transform 0.3s ease, box-shadow 0.3s ease;
}

.card a {
    text-decoration: none;
}

.card-title {
    color: #007bff;
    /* Bootstrap primary color */
}

.grid-container {
    display: grid;
    grid-template-columns: repeat(auto-fit, minmax(350px, 400px));
    gap: 25px;
    padding: 10px;
}

.navbar-nav {
    font-size: larger;
}

.navbar-nav li {
    margin-right: 10px;
    margin-left: 10px;
}

.canvas {
    height: 70vh;
    overflow: hidden;
    overflow-y: scroll;
    background: #e8e8cd;
    background-size: 20px 20px;
    background-image: radial-gradient(circle, cadetblue 1px, rgba(0, 0, 0, 0) 1px);
}

.canvas content {
    margin-top: 10px;
}

body {
    padding-top: 100px;
    background-color: #f8f9fa;
    /* Light grey background */
    font-family: 'Segoe UI', Tahoma, Geneva, Verdana, sans-serif;
}

/*.run-panel-row-height {*/
/*    height: 80vh;*/
/*}*/
pre {
    tab-size: 4;
}

.scroll-column {
    height: 90vh;
    overflow: hidden;
    overflow-y: scroll;
}

label {
    display: block
}

.scroll {
    overflow: auto;
    -webkit-overflow-scrolling: touch;
    /* enables momentum-scrolling on iOS */
    position: absolute;
    top: 0;
    left: 0;
    right: 0;
    bottom: 0;
}


/*Remove the scrollbar from Chrome, Safari, Edge and IE*/
::-webkit-scrollbar {
    background: transparent;
}

* {
    -ms-overflow-style: none !important;
}

.script-table {
    background: white;
    width: fit-content;
}

.script-table td {
    border-bottom: none;
    border-top: cadetblue;
}

.script-table th {
    border-top: cadetblue;
    border-bottom: none;
}

.bottom-button {
    position: absolute;
    bottom: 20px;
}

hr.vertical {
    width: 5px;
    height: 100%;
    display: inline-block;
    /* or height in PX */
}

.list-group a {
    text-align: left;
}

.tray input[type="checkbox"]:checked+label {
    background: radial-gradient(circle, white 40%, midnightblue, dodgerblue 43%);
}

.btn-vial {
    height: 50px;
    width: 50px;
    border-radius: 50%;
    background: lightgray;
    margin-right: 10px;
    margin-top: 10px;
    /*text-align: center;*/

}

.tray {
    width: 70px;
    height: 70px;
    display: inline-block;
    background: darkgrey;
    /*text-align:center;*/
    vertical-align: middle;
}

.disabled-link {
    pointer-events: none;
    color: currentColor;
    cursor: not-allowed;
    opacity: 0.5;
    text-decoration: none;
}


.controller-card a {
    text-decoration: none;
}

#logging-panel {
    flex-grow: 1;
    height: 50vh;
    overflow-y: auto;
    background-color: #f5f5f5;
    padding: 10px;
}

.dropdown:hover .dropdown-menu {
    display: block;
}

#reorder {
    overflow-y: scroll;
    -webkit-overflow-scrolling: touch;
}

.accordion-item.design-control .accordion-button.collapsed {
    background-color: #c1f2f1 !important;
}

.accordion-item.design-control .accordion-button {
    background-color: #b3dad9 !important;
}

.accordion-item.design-control .accordion-button:hover {
    background-color: #b3dad9 !important;
}

.accordion-item.text-to-code .accordion-button.collapsed {
    background-color: #cdc1f2 !important;
}

.accordion-item.text-to-code .accordion-button {
    background-color: #cdc1f2 !important;
}

.accordion-item.text-to-code .accordion-button:hover {
    background-color: #b8afdc !important;
}

.overlay {
    position: fixed;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
    background-color: rgba(0, 0, 0, 0.5);
    display: none;
    z-index: 1000;
    text-align: center;
    color: white;
    font-size: 24px;
    padding-top: 20%;
}

.drop-placeholder {
    height: 2px !important;
    /* Keep it very thin */
    min-height: 2px !important;
    margin: 0 !important;
    padding: 0 !important;
    background: rgba(0, 0, 0, 0.2);
    /* Slight visibility */
    border-radius: 2px;
    list-style: none;
    /* Remove any default list styling */
}

.description-truncate {
    max-width: 600px;
    /* adjust for your layout */
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
}