{# Instrument panel component #}
<div>
    <div class="d-flex justify-content-between align-items-center " style="margin-bottom: 1vh;margin-top: 1vh;">
{#        <a class="btn btn-primary" role="button" type="button" href="{{url_for('design.experiment_builder')}}"><i class="bi bi-arrow-return-left"></i></a>#}
        <a type="button"
                data-get-url="{{ url_for('design.get_operation_sidebar') }}"
                onclick="updateInstrumentPanel(this)">
            <i class="bi bi-arrow-return-left"></i>
        </a>
        {{ instrument | format_name }}
    </div>

    {% if script.editing_type == "script" %}
        {# Auto Fill Toggle #}
        {% include 'components/autofill_toggle.html' %}
    {% endif %}

    {# accordion for instrument #}
    <div class="accordion accordion-flush" id="accordionActions" >

        {% for name, form in forms.items() %}
            {% include 'components/action_form.html' %}
        {% endfor %}
    </div>
</div> 