{# Rename modal component #}
<div class="modal fade" id="renameModal" tabindex="-1" aria-labelledby="renameModal" aria-hidden="true" >
    <div class="modal-dialog">
        <div class="modal-content">
            <div class="modal-header">
                <h1 class="modal-title fs-5" id="renameModal">Rename your script</h1>
                <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>
            </div>
            <form name="run_name" onsubmit="editScriptName(event)">
            <div class="modal-body">
                    <div class="input-group mb-3">
                        <label class="input-group-text" for="new-name">Run Name</label>
                        <input class="form-control" type="text" name="new-name" id="new-name" value="{{ script['name'] }}" required="required">
                    </div>
                </div>
                <div class="modal-footer">
                    <button type="button" class="btn btn-secondary" data-bs-dismiss="modal"> Close </button>
                    <button type="submit" class="btn btn-primary"> Save</button>
                </div>
            </form>
        </div>
    </div>
</div> 