{% extends 'base.html' %}
{% block title %}IvoryOS | Login{% endblock %}


{% block body %}
    <div class= "login">
        <div class="bg-white rounded shadow-sm flex-fill">
            <div class="p-4" style="align-items: center">
                <h5>Log in</h5>
                <form role="form" method='POST' name="login" action="{{ url_for('auth.login') }}">
                    <div class="input-group mb-3">
                        <label class="input-group-text" for="username">Username</label>
                        <input class="form-control" type="text" id="username" name="username">
                    </div>
                    <div class="input-group mb-3">
                        <label class="input-group-text" for="password">Password</label>
                        <input class="form-control" type="password" id="password" name="password">
                    </div>
                    <button type="submit" class="btn btn-secondary" name="login" style="width: 100%;">login</button>
                </form>
                <p class="message">Not registered? <a href="{{ url_for('auth.signup') }}">Create a new account</a>
            </div>
        </div>
    </div>
{% endblock %}
