{# Modals component for experiment builder #}
{% include 'components/modals/new_script_modal.html' %}
{% include 'components/modals/saveas_modal.html' %}
{% include 'components/modals/rename_modal.html' %}
{% include 'components/modals/json_modal.html' %}
{% include 'components/modals/python_import_modal.html' %}
{% include 'components/modals/drop_modal.html' %}