 {% include 'components/canvas_header.html' %}
     <div class="canvas canvas-action-wrapper position-relative" id="canvas-action-wrapper" droppable="true">
         {% include 'components/canvas_main.html' %}
     </div>
 {% include 'components/canvas_footer.html' %}