{# Progress panel component for experiment run #}
<div class="col-lg-6 col-sm-12" id="code-panel">
    <p>
    <h5>Progress：</h5>
    {% from 'components/render_node_macro.html' import render_nodes %}

    {% if "prep" in line_collection.keys() and line_collection["prep"] %}
    <h6>Preparation：</h6>
    {{ render_nodes(line_collection["prep"]) }}
    {% endif %}

    {% if "script" in line_collection.keys() and line_collection["script"] %}
    <h6>Experiment：</h6>
    {{ render_nodes(line_collection["script"]) }}
    {% endif %}

    {% if "cleanup" in line_collection.keys() and line_collection["cleanup"] %}
    <h6>Cleanup：</h6>
    {{ render_nodes(line_collection["cleanup"]) }}
    {% endif %}
    </p>
</div>