{# Canvas footer component #}
<div>
    <a class="btn btn-dark {{ 'disabled' if not script.name or script.status == "finalized" else '' }}"
       href="#" data-post-url="{{ url_for('library.workflow_script', script_name=script.name or 'untitled') }}"
       onclick="saveWorkflow(this); return false;">
        Quick Save
    </a>
    <a class="btn btn-dark " href="{{ url_for('execute.experiment_run') }}">Prepare Run</a>
    <span class="text-muted small ms-2">
        <i class="bi bi-info-circle"></i>
        Does not start the workflow
    </span>
</div> 